/***************************************************************************
 *   Copyright (C) 2010-2011 Alejandro Fiestas Olivares <afiestas@kde.org> *
 *   Copyright (C) 2010-2011 UFO Coders <info@ufocoders.com>               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA            *
 ***************************************************************************/

#include "linkproxymodel.h"
#include <QDebug>
#include <QModelIndex>
#include <QFileSystemModel>

bool LinkProxyModel::filterAcceptsRow(int source_row, const QModelIndex& source_parent) const
{
    QFileSystemModel *model = qobject_cast< QFileSystemModel *>(sourceModel());
    QModelIndex in = model->index(source_row, source_parent.column(), source_parent);
    const QString path = in.data(QFileSystemModel::FilePathRole).toString();

    if (path == model->rootPath()) {
        return true;
    }

    QFileInfo file(path);
    return file.isSymLink();
}
